pub struct ExpansionInfo<'db> {
pub(crate) expanded: InMacroFile<SyntaxNode>,
pub(crate) arg: InFile<Option<SyntaxNode>>,
pub(crate) exp_map: &'db ExpansionSpanMap,
pub(crate) arg_map: SpanMap<'db>,
pub(crate) loc: &'db MacroCallLoc,
}Expand description
ExpansionInfo mainly describes how to map text range between src and expanded macro
Fields
expanded: InMacroFile<SyntaxNode>arg: InFile<Option<SyntaxNode>>The argument TokenTree or item for attributes
exp_map: &'db ExpansionSpanMaparg_map: SpanMap<'db>loc: &'db MacroCallLocImplementations
Sourceimpl<'db> ExpansionInfo<'db>
impl<'db> ExpansionInfo<'db>
pub fn expanded(&self) -> InMacroFile<SyntaxNode>
pub fn arg(&self) -> InFile<Option<&SyntaxNode>>
pub fn call_file(&self) -> HirFileId
pub fn is_attr(&self) -> bool
Sourcepub fn map_range_down_exact(
&self,
span: Span,
) -> Option<InMacroFile<impl Iterator<Item = (SyntaxToken, SyntaxContext)> + '_>>
pub fn map_range_down_exact( &self, span: Span, ) -> Option<InMacroFile<impl Iterator<Item = (SyntaxToken, SyntaxContext)> + '_>>
Maps the passed in file range down into a macro expansion if it is the input to a macro call.
Note this does a linear search through the entire backing vector of the spanmap.
Sourcepub fn map_range_down(
&self,
span: Span,
) -> Option<InMacroFile<impl Iterator<Item = (SyntaxToken, SyntaxContext)> + '_>>
pub fn map_range_down( &self, span: Span, ) -> Option<InMacroFile<impl Iterator<Item = (SyntaxToken, SyntaxContext)> + '_>>
Maps the passed in file range down into a macro expansion if it is the input to a macro call.
Unlike ExpansionInfo::map_range_down_exact, this will consider spans that contain the given span.
Note this does a linear search through the entire backing vector of the spanmap.
Sourcepub fn span_for_offset(
&self,
db: &dyn SourceDatabase,
offset: TextSize,
) -> (FileRange, SyntaxContext)
pub fn span_for_offset( &self, db: &dyn SourceDatabase, offset: TextSize, ) -> (FileRange, SyntaxContext)
Looks up the span at the given offset.
Sourcepub fn map_node_range_up(
&self,
db: &dyn SourceDatabase,
range: TextRange,
) -> Option<(FileRange, SyntaxContext)>
pub fn map_node_range_up( &self, db: &dyn SourceDatabase, range: TextRange, ) -> Option<(FileRange, SyntaxContext)>
Maps up the text range out of the expansion hierarchy back into the original file its from.
Sourcepub fn map_range_up_once(
&self,
db: &dyn SourceDatabase,
token: TextRange,
) -> InFile<SmallVec<[TextRange; 1]>>
pub fn map_range_up_once( &self, db: &dyn SourceDatabase, token: TextRange, ) -> InFile<SmallVec<[TextRange; 1]>>
Maps up the text range out of the expansion into its macro call.
Note that this may return multiple ranges as we lose the precise association between input to output and as such we may consider inputs that are unrelated.
pub fn new( db: &'db dyn SourceDatabase, macro_file: MacroCallId, ) -> ExpansionInfo<'db>