pub(crate) type MacroArgResult = (TopSubtree, SyntaxFixupUndoInfo, Span);Expand description
This is just to ensure the types of MacroCallId::macro_arg_considering_derives
and MacroCallId::macro_arg are the same.
pub(crate) type MacroArgResult = (TopSubtree, SyntaxFixupUndoInfo, Span);This is just to ensure the types of MacroCallId::macro_arg_considering_derives
and MacroCallId::macro_arg are the same.