pub struct MacroCallId(pub(crate) Id, pub(crate) PhantomData<fn() -> &'static ()>);Expand description
Macro ids. That’s probably the tricksiest bit in rust-analyzer, and the reason why we use salsa at all.
We encode macro definitions into ids of macro calls, this what allows us to be incremental.
Tuple Fields
0: Id1: PhantomData<fn() -> &'static ()>Implementations
Sourceimpl MacroCallId
impl MacroCallId
pub fn expansion_span_map(self, db: &dyn SourceDatabase) -> &ExpansionSpanMap
Sourceimpl MacroCallId
impl MacroCallId
pub fn call_node(self, db: &dyn SourceDatabase) -> InFile<SyntaxNode>
pub fn expansion_level(self, db: &dyn SourceDatabase) -> u32
pub fn parent(self, db: &dyn SourceDatabase) -> HirFileId
Sourcepub fn expansion_info(self, db: &dyn SourceDatabase) -> ExpansionInfo<'_>
pub fn expansion_info(self, db: &dyn SourceDatabase) -> ExpansionInfo<'_>
Return expansion information if it is a macro-expansion file
pub fn kind(self, db: &dyn SourceDatabase) -> MacroKind
pub fn is_include_macro(self, db: &dyn SourceDatabase) -> bool
pub fn is_include_like_macro(self, db: &dyn SourceDatabase) -> bool
pub fn is_env_or_option_env(self, db: &dyn SourceDatabase) -> bool
pub fn is_eager(self, db: &dyn SourceDatabase) -> bool
pub fn eager_arg(self, db: &dyn SourceDatabase) -> Option<MacroCallId>
pub fn is_derive_attr_pseudo_expansion(self, db: &dyn SourceDatabase) -> bool
Sourceimpl MacroCallId
impl MacroCallId
Sourcepub fn parse_macro_expansion<'db>(
self: MacroCallId,
db: &'db dyn SourceDatabase,
) -> &'db ExpandResult<(Parse<SyntaxNode>, ExpansionSpanMap)>
pub fn parse_macro_expansion<'db>( self: MacroCallId, db: &'db dyn SourceDatabase, ) -> &'db ExpandResult<(Parse<SyntaxNode>, ExpansionSpanMap)>
Implementation of HirFileId::parse_or_expand for the macro case.
pub fn parse_macro_expansion_error( self, db: &dyn SourceDatabase, ) -> Option<ExpandResult<Arc<[SyntaxError]>>>
Sourcepub fn macro_arg_considering_derives<'db>(
self,
db: &'db dyn SourceDatabase,
kind: &MacroCallKind,
) -> &'db (TopSubtree, SyntaxFixupUndoInfo, Span)
pub fn macro_arg_considering_derives<'db>( self, db: &'db dyn SourceDatabase, kind: &MacroCallKind, ) -> &'db (TopSubtree, SyntaxFixupUndoInfo, Span)
This resolves the MacroCallId to check if it is a derive macro if so get the macro_arg for the derive. Other wise return the macro_arg for the macro_call_id.
This is not connected to the database so it does not cache the result. However, the inner macro_arg query is
Sourcepub(crate) fn macro_arg<'db>(
self: MacroCallId,
db: &'db dyn SourceDatabase,
) -> &'db (TopSubtree, SyntaxFixupUndoInfo, Span)
pub(crate) fn macro_arg<'db>( self: MacroCallId, db: &'db dyn SourceDatabase, ) -> &'db (TopSubtree, SyntaxFixupUndoInfo, Span)
Lowers syntactic macro call to a token tree representation. That’s a firewall query, only typing in the macro call itself changes the returned subtree.
pub(crate) fn macro_expand<'db>( self, db: &'db dyn SourceDatabase, loc: &MacroCallLoc, ) -> ExpandResult<(Cow<'db, TopSubtree>, MatchedArmIndex)>
Sourcepub(crate) fn expand_proc_macro<'db>(
self: MacroCallId,
db: &'db dyn SourceDatabase,
) -> &'db ExpandResult<TopSubtree>
pub(crate) fn expand_proc_macro<'db>( self: MacroCallId, db: &'db dyn SourceDatabase, ) -> &'db ExpandResult<TopSubtree>
Special case of Self::macro_expand for procedural macros. We can’t LRU
proc macros, since they are not deterministic in general, and
non-determinism breaks salsa in a very, very, very bad way.
@edwin0cheng heroically debugged this once! See #4315 for details
Sourceimpl MacroCallId
impl MacroCallId
Sourcepub fn expand_speculative(
self,
db: &dyn SourceDatabase,
speculative_args: &SyntaxNode,
token_to_map: SyntaxToken,
) -> Option<(SyntaxNode, Vec<(SyntaxToken, u8)>)>
pub fn expand_speculative( self, db: &dyn SourceDatabase, speculative_args: &SyntaxNode, token_to_map: SyntaxToken, ) -> Option<(SyntaxNode, Vec<(SyntaxToken, u8)>)>
This expands the given macro call, but with different arguments. This is
used for completion, where we want to see what ‘would happen’ if we insert a
token. The token_to_map mapped down into the expansion, with the mapped
token(s) returned with their priority.
Sourceimpl MacroCallId
impl MacroCallId
pub fn ingredient(zalsa: &Zalsa) -> &IngredientImpl<Self>
Sourceimpl<'db> MacroCallId
impl<'db> MacroCallId
pub fn new<Db_, T0: Lookup<MacroCallLoc> + Hash>(db: &'db Db_, loc: T0) -> Selfwhere
Db_: ?Sized + Database,
MacroCallLoc: HashEqLike<T0>,
pub fn loc<Db_>(self, db: &'db Db_) -> &'db MacroCallLocwhere
Db_: ?Sized + Database,
Sourceimpl MacroCallId
impl MacroCallId
Sourcepub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> MacroCallLoc: Debug,
pub fn default_debug_fmt(this: Self, f: &mut Formatter<'_>) -> Resultwhere
for<'db> MacroCallLoc: Debug,
Default debug formatting for this struct (may be useful if you define your own Debug impl)